module Nx
  class DoubanMovie
    def initialize(url)
      @doc = Nokogiri::HTML(URI.open(url))
      @url = url
      @info_list = @doc.css("#info .pl")
    end

    def url
      @url
    end

    def to_h
      methods = (public_methods(false) - [__method__])
      methods.map do |method|
        [method, send(method)]
      end.to_h
    end

    # 电影标题
    def title
      el = @doc.css('#content > h1 span[property="v:itemreviewed"]')
      el.text
    end

    # 豆瓣成员常用的标签
    def tags
      els = @doc.css(".tags .tags-body a")
      els.map do |el|
        el.text
      end
    end

    # 演职员
    def celebrities
      els = @doc.css("#celebrities .celebrity")
      els.map do |el|
        anchor = el.at_css("a")
        info = el.at_css(".info")

        {
          title: anchor.attribute("title").value,
          url: anchor.attribute("href").value,
          avatar: style_bg(anchor.at_css(".avatar").attribute("style").value),
          role: info.at_css(".role").text,
          name: info.at_css(".name").text,
        }
      end
    end

    # 电影评分
    def rate
      el = @doc.css('#interest_sectl [property="v:average"]')
      el.text
    end

    # 剧情简介
    def summary
      el = @doc.css('#link-report [property="v:summary"]')
      clean_string el.text
    end

    # 上映年份
    def year
      el = @doc.css('#content > h1 span[class="year"]')
      el.text.tr("()", "")
    end

    # 封面
    def cover
      el = @doc.css("#mainpic .nbgnbg img")
      el.attribute("src").value
    end

    # 导演
    def director
      els = @doc.css('#info .attrs [rel="v:directedBy"]')
      els.map do |el|
        url = el.attribute("href").value
        {
          url: "https://movie.douban.com#{url}",
          name: el.text,
        }
      end
    end

    # 编剧
    def screenwriter
      el = @info_list.find do |item|
        item.text.include? "编剧"
      end
      clean_string el.nil? ? "" : el.next_element.text
    end

    # 主演
    def leading_roles
      els = @doc.css('#info .actor .attrs a[rel="v:starring"]')
      els.map do |el|
        url = el.attribute("href").value
        {
          url: "https://movie.douban.com#{url}",
          name: el.text,
        }
      end
    end

    # 类型
    def types
      els = @doc.css('#info span[property="v:genre"]')
      els.map do |el|
        el.text
      end
    end

    # 制片国家/地区
    def district
      el = @info_list.find do |item|
        item.text.include? "制片国家/地区"
      end
      clean_string el.nil? ? "" : el.next.text
    end

    # 语言:
    def language
      el = @info_list.find do |item|
        item.text.include? "语言"
      end
      clean_string el.nil? ? "" : el.next.text
    end

    # 上映日期:
    def release_dates
      els = @doc.css('#info [property="v:initialReleaseDate"]')
      els.map do |el|
        clean_string el.text
      end
    end

    # 片长:
    def runtime
      el = @doc.css('#info [property="v:runtime"]')
      clean_string(el.text)
    end

    # 又名:
    def aliases
      el = @info_list.find do |item|
        item.text.include? "又名"
      end
      clean_string el.nil? ? "" : el.next.text
    end

    # IMDb链接:
    def imdb_url
      el = @info_list.find do |item|
        item.text.include? "IMDb链接"
      end

      el.next_element.attribute("href").value
    end

    private

    def clean_string(str)
      str.strip
    end

    def style_bg(str)
      matched = /url\((.*)\)/.match(str)
      matched.nil? ? "" : matched[1]
    end
  end
end
